@@ -28,22 +28,23 @@
 #include <fcntl.h>
 #include <fnmatch.h>
 #include <getopt.h>
+#include <libevdev/libevdev.h>
+#include <libudev.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <libudev.h>
 #include <unistd.h>
 
-#include <libevdev/libevdev.h>
-
-#include "builddir.h"
-#include "libinput.h"
-#include "shared.h"
 #include "util-macros.h"
 #include "util-strings.h"
 
+#include "builddir.h"
+#include "libinput-util.h"
+#include "libinput.h"
+#include "shared.h"
+
 static uint32_t dispatch_counter = 0;
 uint32_t log_serial = 0;
 
@@ -77,13 +78,11 @@ log_handler(struct libinput *li,
 			else
 				printf(ANSI_RED);
 		} else if (priority >= LIBINPUT_LOG_PRIORITY_INFO) {
-			printf(ANSI_HIGHLIGHT);
+			printf(ANSI_BOLD);
 		} else if (priority == LIBINPUT_LOG_PRIORITY_DEBUG) {
 			if (dispatch_counter != last_dispatch_no)
 				color_toggle = !color_toggle;
-			uint8_t r = 0,
-				g = 135,
-				b = 95 + (color_toggle ? 80 :0);
+			uint8_t r = 0, g = 135, b = 95 + (color_toggle ? 80 : 0);
 			printf("\x1B[38;2;%u;%u;%um", r, g, b);
 		}
 	}
@@ -124,7 +123,7 @@ tools_init_options(struct tools_options *options)
 	options->speed = 0.0;
 	options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_NONE;
 	/* initialize accel args */
-	static double points[] = {0.0, 1.0};
+	static double points[] = { 0.0, 1.0 };
 	options->custom_points = points;
 	options->custom_npoints = ARRAY_LENGTH(points);
 	options->custom_type = LIBINPUT_ACCEL_TYPE_FALLBACK;
@@ -138,14 +137,15 @@ tools_init_options(struct tools_options *options)
 	options->area.x2 = 1.0;
 	options->area.y2 = 1.0;
 	options->sendevents = LIBINPUT_CONFIG_SEND_EVENTS_ENABLED;
+	options->eraser_button_mode = LIBINPUT_CONFIG_ERASER_BUTTON_DEFAULT;
+	options->eraser_button_button = BTN_STYLUS;
+	options->eraser_button_button = 0;
 }
 
 int
-tools_parse_option(int option,
-		   const char *optarg,
-		   struct tools_options *options)
+tools_parse_option(int option, const char *optarg, struct tools_options *options)
 {
-	switch(option) {
+	switch (option) {
 	case OPT_TAP_ENABLE:
 		options->tapping = 1;
 		break;
@@ -173,9 +173,11 @@ tools_parse_option(int option,
 	case OPT_DRAG_LOCK_ENABLE:
 		if (optarg) {
 			if (streq(optarg, "sticky")) {
-				options->drag_lock = LIBINPUT_CONFIG_DRAG_LOCK_ENABLED_STICKY;
+				options->drag_lock =
+					LIBINPUT_CONFIG_DRAG_LOCK_ENABLED_STICKY;
 			} else if (streq(optarg, "timeout")) {
-				options->drag_lock = LIBINPUT_CONFIG_DRAG_LOCK_ENABLED_TIMEOUT;
+				options->drag_lock =
+					LIBINPUT_CONFIG_DRAG_LOCK_ENABLED_TIMEOUT;
 			} else {
 				return 1;
 			}
@@ -221,14 +223,13 @@ tools_parse_option(int option,
 			return 1;
 
 		if (streq(optarg, "none")) {
-			options->click_method =
-			LIBINPUT_CONFIG_CLICK_METHOD_NONE;
+			options->click_method = LIBINPUT_CONFIG_CLICK_METHOD_NONE;
 		} else if (streq(optarg, "clickfinger")) {
 			options->click_method =
-			LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
+				LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
 		} else if (streq(optarg, "buttonareas")) {
 			options->click_method =
-			LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
+				LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
 		} else {
 			return 1;
 		}
@@ -250,17 +251,13 @@ tools_parse_option(int option,
 			return 1;
 
 		if (streq(optarg, "none")) {
-			options->scroll_method =
-			LIBINPUT_CONFIG_SCROLL_NO_SCROLL;
+			options->scroll_method = LIBINPUT_CONFIG_SCROLL_NO_SCROLL;
 		} else if (streq(optarg, "twofinger")) {
-			options->scroll_method =
-			LIBINPUT_CONFIG_SCROLL_2FG;
+			options->scroll_method = LIBINPUT_CONFIG_SCROLL_2FG;
 		} else if (streq(optarg, "edge")) {
-			options->scroll_method =
-			LIBINPUT_CONFIG_SCROLL_EDGE;
+			options->scroll_method = LIBINPUT_CONFIG_SCROLL_EDGE;
 		} else if (streq(optarg, "button")) {
-			options->scroll_method =
-			LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN;
+			options->scroll_method = LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN;
 		} else {
 			return 1;
 		}
@@ -269,13 +266,9 @@ tools_parse_option(int option,
 		if (!optarg) {
 			return 1;
 		}
-		options->scroll_button =
-		libevdev_event_code_from_name(EV_KEY,
-					      optarg);
+		options->scroll_button = libevdev_event_code_from_name(EV_KEY, optarg);
 		if (options->scroll_button == -1) {
-			fprintf(stderr,
-				"Invalid button %s\n",
-				optarg);
+			fprintf(stderr, "Invalid button %s\n", optarg);
 			return 1;
 		}
 		break;
@@ -297,11 +290,11 @@ tools_parse_option(int option,
 		if (streq(optarg, "adaptive"))
 			options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE;
 		else if (streq(optarg, "flat"))
-		      options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT;
+			options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT;
 		else if (streq(optarg, "custom"))
-		      options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM;
+			options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM;
 		else
-		      return 1;
+			return 1;
 		break;
 	case OPT_DISABLE_SENDEVENTS:
 		if (!optarg)
@@ -318,7 +311,8 @@ tools_parse_option(int option,
 		else if (streq(optarg, "enabled"))
 			options->sendevents = LIBINPUT_CONFIG_SEND_EVENTS_ENABLED;
 		else if (streq(optarg, "disabled-on-external-mouse"))
-			options->sendevents = LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE;
+			options->sendevents =
+				LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE;
 		else {
 			fprintf(stderr, "Invalid sendevents mode: %s\n", optarg);
 			return 1;
@@ -328,17 +322,13 @@ tools_parse_option(int option,
 		if (!optarg)
 			return 1;
 
-		snprintf(options->match,
-			 sizeof(options->match),
-			 "%s",
-			 optarg);
+		snprintf(options->match, sizeof(options->match), "%s", optarg);
 		break;
 	case OPT_CUSTOM_POINTS:
 		if (!optarg)
 			return 1;
-		options->custom_points = double_array_from_string(optarg,
-								  ";",
-								  &options->custom_npoints);
+		options->custom_points =
+			double_array_from_string(optarg, ";", &options->custom_npoints);
 		if (!options->custom_points || options->custom_npoints < 2) {
 			fprintf(stderr,
 				"Invalid --set-custom-points\n"
@@ -362,8 +352,9 @@ tools_parse_option(int option,
 		else if (streq(optarg, "scroll"))
 			options->custom_type = LIBINPUT_ACCEL_TYPE_SCROLL;
 		else {
-			fprintf(stderr, "Invalid --set-custom-type\n"
-			                "Valid custom types: fallback|motion|scroll\n");
+			fprintf(stderr,
+				"Invalid --set-custom-type\n"
+				"Valid custom types: fallback|motion|scroll\n");
 			return 1;
 		}
 		break;
@@ -381,31 +372,32 @@ tools_parse_option(int option,
 			return 1;
 
 		size_t npoints = 0;
-		double *range = double_array_from_string(optarg, ":", &npoints);
-		if (npoints != 2 || !range || range[0] < 0.0 || range[1] > 1.0 || range[0] >= range[1]) {
-			free(range);
-			fprintf(stderr, "Invalid pressure range, must be in format \"min:max\"\n");
+		_autofree_ double *range =
+			double_array_from_string(optarg, ":", &npoints);
+		if (npoints != 2 || !range || range[0] < 0.0 || range[1] > 1.0 ||
+		    range[0] >= range[1]) {
+			fprintf(stderr,
+				"Invalid pressure range, must be in format \"min:max\"\n");
 			return 1;
 		}
 		options->pressure_range[0] = range[0];
 		options->pressure_range[1] = range[1];
-		free(range);
 		break;
-		}
+	}
 	case OPT_CALIBRATION: {
 		if (!optarg)
 			return 1;
 
 		size_t npoints = 0;
-		double *matrix = double_array_from_string(optarg, " ", &npoints);
+		_autofree_ double *matrix =
+			double_array_from_string(optarg, " ", &npoints);
 		if (!matrix || npoints != 6) {
-			free(matrix);
-			fprintf(stderr, "Invalid calibration matrix, must be 6 space-separated values\n");
+			fprintf(stderr,
+				"Invalid calibration matrix, must be 6 space-separated values\n");
 			return 1;
 		}
 		for (size_t i = 0; i < 6; i++)
-			options->calibration[i] =  matrix[i];
-		free(matrix);
+			options->calibration[i] = matrix[i];
 		break;
 	}
 	case OPT_AREA: {
@@ -423,7 +415,7 @@ tools_parse_option(int option,
 		options->area.x2 = x2;
 		options->area.y2 = y2;
 		break;
-		}
+	}
 	case OPT_3FG_DRAG:
 		if (!optarg)
 			return 1;
@@ -434,8 +426,39 @@ tools_parse_option(int option,
 		else if (streq(optarg, "disabled"))
 			options->drag_3fg = LIBINPUT_CONFIG_3FG_DRAG_DISABLED;
 		else {
-			fprintf(stderr, "Invalid --enable-3fg-drag\n"
-			                "Valid options: 3fg|4fg|disabled\n");
+			fprintf(stderr,
+				"Invalid --enable-3fg-drag\n"
+				"Valid options: 3fg|4fg|disabled\n");
+			return 1;
+		}
+		break;
+	case OPT_ERASER_BUTTON_MODE:
+		if (!optarg)
+			return 1;
+		if (streq(optarg, "default"))
+			options->eraser_button_mode =
+				LIBINPUT_CONFIG_ERASER_BUTTON_DEFAULT;
+		else if (streq(optarg, "button"))
+			options->eraser_button_mode =
+				LIBINPUT_CONFIG_ERASER_BUTTON_BUTTON;
+		else {
+			fprintf(stderr,
+				"Invalid --set-eraser-button-mode\n"
+				"Valid options: default|button\n");
+			return 1;
+		}
+		break;
+	case OPT_ERASER_BUTTON_BUTTON:
+		if (!optarg)
+			return 1;
+		if (streq(optarg, "BTN_STYLUS"))
+			options->eraser_button_button = BTN_STYLUS;
+		else if (streq(optarg, "BTN_STYLUS2"))
+			options->eraser_button_button = BTN_STYLUS2;
+		else if (streq(optarg, "BTN_STYLUS3"))
+			options->eraser_button_button = BTN_STYLUS3;
+		else {
+			fprintf(stderr, "Unsupported eraser button %s\n", optarg);
 			return 1;
 		}
 		break;
@@ -450,11 +473,12 @@ open_restricted(const char *path, int flags, void *user_data)
 	int fd = open(path, flags);
 
 	if (fd < 0)
-		fprintf(stderr, "Failed to open %s (%s)\n",
-			path, strerror(errno));
-	else if (grab && *grab && ioctl(fd, EVIOCGRAB, (void*)1) == -1)
-		fprintf(stderr, "Grab requested, but failed for %s (%s)\n",
-			path, strerror(errno));
+		fprintf(stderr, "Failed to open %s (%s)\n", path, strerror(errno));
+	else if (grab && *grab && ioctl(fd, EVIOCGRAB, (void *)1) == -1)
+		fprintf(stderr,
+			"Grab requested, but failed for %s (%s)\n",
+			path,
+			strerror(errno));
 
 	return fd < 0 ? -errno : fd;
 }
@@ -473,18 +497,16 @@ static const struct libinput_interface interface = {
 static struct libinput *
 tools_open_udev(const char *seat, bool verbose, bool *grab)
 {
-	struct libinput *li;
-	struct udev *udev = udev_new();
-
+	_unref_(udev) *udev = udev_new();
 	if (!udev) {
 		fprintf(stderr, "Failed to initialize udev\n");
 		return NULL;
 	}
 
-	li = libinput_udev_create_context(&interface, grab, udev);
+	_unref_(libinput) *li = libinput_udev_create_context(&interface, grab, udev);
 	if (!li) {
 		fprintf(stderr, "Failed to initialize context from udev\n");
-		goto out;
+		return NULL;
 	}
 
 	libinput_log_set_handler(li, log_handler);
@@ -493,24 +515,16 @@ tools_open_udev(const char *seat, bool verbose, bool *grab)
 
 	if (libinput_udev_assign_seat(li, seat)) {
 		fprintf(stderr, "Failed to set seat\n");
-		libinput_unref(li);
-		li = NULL;
-		goto out;
+		return NULL;
 	}
 
-out:
-	udev_unref(udev);
-	return li;
+	return steal(&li);
 }
 
 static struct libinput *
 tools_open_device(const char **paths, bool verbose, bool *grab)
 {
-	struct libinput_device *device;
-	struct libinput *li;
-	const char **p = paths;
-
-	li = libinput_path_create_context(&interface, grab);
+	_unref_(libinput) *li = libinput_path_create_context(&interface, grab);
 	if (!li) {
 		fprintf(stderr, "Failed to initialize path context\n");
 		return NULL;
@@ -521,28 +535,24 @@ tools_open_device(const char **paths, bool verbose, bool *grab)
 		libinput_log_set_priority(li, LIBINPUT_LOG_PRIORITY_DEBUG);
 	}
 
+	const char **p = paths;
 	while (*p) {
-		device = libinput_path_add_device(li, *p);
+		struct libinput_device *device = libinput_path_add_device(li, *p);
 		if (!device) {
 			fprintf(stderr, "Failed to initialize device %s\n", *p);
-			libinput_unref(li);
-			li = NULL;
-			break;
+			return NULL;
 		}
 		p++;
 	}
 
-	return li;
+	return steal(&li);
 }
 
 static void
 tools_setenv_quirks_dir(void)
 {
-	char *builddir = builddir_lookup();
-	if (builddir) {
+	if (builddir_lookup(NULL))
 		setenv("LIBINPUT_QUIRKS_DIR", LIBINPUT_QUIRKS_SRCDIR, 0);
-		free(builddir);
-	}
 }
 
 struct libinput *
@@ -570,16 +580,16 @@ tools_open_backend(enum tools_backend which,
 }
 
 void
-tools_device_apply_config(struct libinput_device *device,
-			  struct tools_options *options)
+tools_device_apply_config(struct libinput_device *device, struct tools_options *options)
 {
 	const char *name = libinput_device_get_name(device);
 
 	if (libinput_device_config_send_events_get_modes(device) &
-	      LIBINPUT_CONFIG_SEND_EVENTS_DISABLED &&
+		    LIBINPUT_CONFIG_SEND_EVENTS_DISABLED &&
 	    fnmatch(options->disable_pattern, name, 0) != FNM_NOMATCH) {
-		libinput_device_config_send_events_set_mode(device,
-					    LIBINPUT_CONFIG_SEND_EVENTS_DISABLED);
+		libinput_device_config_send_events_set_mode(
+			device,
+			LIBINPUT_CONFIG_SEND_EVENTS_DISABLED);
 	}
 
 	if (strlen(options->match) > 0 &&
@@ -590,23 +600,23 @@ tools_device_apply_config(struct libinput_device *device,
 
 	if (options->tapping != -1)
 		libinput_device_config_tap_set_enabled(device, options->tapping);
-	if (options->tap_map != (enum libinput_config_tap_button_map)-1)
-		libinput_device_config_tap_set_button_map(device,
-							  options->tap_map);
+	if (options->tap_map != (enum libinput_config_tap_button_map) - 1)
+		libinput_device_config_tap_set_button_map(device, options->tap_map);
 	if (options->drag != -1)
-		libinput_device_config_tap_set_drag_enabled(device,
-							    options->drag);
+		libinput_device_config_tap_set_drag_enabled(device, options->drag);
 	if (options->drag_lock != -1)
 		libinput_device_config_tap_set_drag_lock_enabled(device,
 								 options->drag_lock);
 	if (options->natural_scroll != -1)
-		libinput_device_config_scroll_set_natural_scroll_enabled(device,
-									 options->natural_scroll);
+		libinput_device_config_scroll_set_natural_scroll_enabled(
+			device,
+			options->natural_scroll);
 	if (options->left_handed != -1)
 		libinput_device_config_left_handed_set(device, options->left_handed);
 	if (options->middlebutton != -1)
-		libinput_device_config_middle_emulation_set_enabled(device,
-								    options->middlebutton);
+		libinput_device_config_middle_emulation_set_enabled(
+			device,
+			options->middlebutton);
 
 	if (options->dwt != -1)
 		libinput_device_config_dwt_set_enabled(device, options->dwt);
@@ -614,48 +624,50 @@ tools_device_apply_config(struct libinput_device *device,
 	if (options->dwtp != -1)
 		libinput_device_config_dwtp_set_enabled(device, options->dwtp);
 
-	if (options->click_method != (enum libinput_config_click_method)-1)
+	if (options->click_method != (enum libinput_config_click_method) - 1)
 		libinput_device_config_click_set_method(device, options->click_method);
 
-	if (options->clickfinger_map != (enum libinput_config_clickfinger_button_map)-1)
-		libinput_device_config_click_set_clickfinger_button_map(device,
-									options->clickfinger_map);
+	if (options->clickfinger_map !=
+	    (enum libinput_config_clickfinger_button_map) - 1)
+		libinput_device_config_click_set_clickfinger_button_map(
+			device,
+			options->clickfinger_map);
 
-	if (options->scroll_method != (enum libinput_config_scroll_method)-1)
+	if (options->scroll_method != (enum libinput_config_scroll_method) - 1)
 		libinput_device_config_scroll_set_method(device,
 							 options->scroll_method);
 	if (options->scroll_button != -1)
 		libinput_device_config_scroll_set_button(device,
 							 options->scroll_button);
 	if (options->scroll_button_lock != -1)
-		libinput_device_config_scroll_set_button_lock(device,
-							      options->scroll_button_lock);
+		libinput_device_config_scroll_set_button_lock(
+			device,
+			options->scroll_button_lock);
 
 	if (libinput_device_config_accel_is_available(device)) {
-		libinput_device_config_accel_set_speed(device,
-						       options->speed);
+		libinput_device_config_accel_set_speed(device, options->speed);
 		if (options->profile != LIBINPUT_CONFIG_ACCEL_PROFILE_NONE)
 			libinput_device_config_accel_set_profile(device,
 								 options->profile);
 	}
 
 	if (options->profile == LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM) {
-		struct libinput_config_accel *config =
-			libinput_config_accel_create(LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM);
+		_destroy_(libinput_config_accel) *config = libinput_config_accel_create(
+			LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM);
 		libinput_config_accel_set_points(config,
 						 options->custom_type,
 						 options->custom_step,
 						 options->custom_npoints,
 						 options->custom_points);
 		libinput_device_config_accel_apply(device, config);
-		libinput_config_accel_destroy(config);
 	}
 
 	if (options->angle != 0)
 		libinput_device_config_rotation_set_angle(device, options->angle % 360);
 
 	if (libinput_device_config_calibration_has_matrix(device))
-		libinput_device_config_calibration_set_matrix(device, options->calibration);
+		libinput_device_config_calibration_set_matrix(device,
+							      options->calibration);
 
 	if (libinput_device_config_area_has_rectangle(device))
 		libinput_device_config_area_set_rectangle(device, &options->area);
@@ -671,62 +683,55 @@ tools_tablet_tool_apply_config(struct libinput_tablet_tool *tool,
 	libinput_tablet_tool_config_pressure_range_set(tool,
 						       options->pressure_range[0],
 						       options->pressure_range[1]);
+	if (options->eraser_button_button)
+		libinput_tablet_tool_config_eraser_button_set_button(
+			tool,
+			options->eraser_button_button);
+	libinput_tablet_tool_config_eraser_button_set_mode(tool,
+							   options->eraser_button_mode);
 }
 
-static char*
+static char *
 find_device(const char *udev_tag)
 {
-	struct udev *udev;
-	struct udev_enumerate *e = NULL;
-	struct udev_list_entry *entry = NULL;
-	struct udev_device *device;
-	const char *path, *sysname;
-	char *device_node = NULL;
-
-	udev = udev_new();
+	_unref_(udev) *udev = udev_new();
 	if (!udev)
-		goto out;
+		return NULL;
 
-	e = udev_enumerate_new(udev);
+	_unref_(udev_enumerate) *e = udev_enumerate_new(udev);
 	udev_enumerate_add_match_subsystem(e, "input");
 	udev_enumerate_scan_devices(e);
 
+	struct udev_list_entry *entry = NULL;
 	udev_list_entry_foreach(entry, udev_enumerate_get_list_entry(e)) {
-		path = udev_list_entry_get_name(entry);
-		device = udev_device_new_from_syspath(udev, path);
+		const char *path = udev_list_entry_get_name(entry);
+		_unref_(udev_device) *device = udev_device_new_from_syspath(udev, path);
 		if (!device)
 			continue;
 
-		sysname = udev_device_get_sysname(device);
+		const char *sysname = udev_device_get_sysname(device);
 		if (!strstartswith("event", sysname)) {
-			udev_device_unref(device);
 			continue;
 		}
 
-		if (udev_device_get_property_value(device, udev_tag))
-			device_node = safe_strdup(udev_device_get_devnode(device));
-
-		udev_device_unref(device);
-
-		if (device_node)
-			break;
+		if (udev_device_get_property_value(device, udev_tag)) {
+			char *device_node =
+				safe_strdup(udev_device_get_devnode(device));
+			if (device_node)
+				return device_node;
+		}
 	}
-out:
-	udev_enumerate_unref(e);
-	udev_unref(udev);
 
-	return device_node;
+	return NULL;
 }
 
 bool
 find_touchpad_device(char *path, size_t path_len)
 {
-	char *devnode = find_device("ID_INPUT_TOUCHPAD");
+	_autofree_ char *devnode = find_device("ID_INPUT_TOUCHPAD");
 
-	if (devnode) {
+	if (devnode)
 		snprintf(path, path_len, "%s", devnode);
-		free(devnode);
-	}
 
 	return devnode != NULL;
 }
@@ -734,29 +739,19 @@ find_touchpad_device(char *path, size_t path_len)
 bool
 is_touchpad_device(const char *devnode)
 {
-	struct udev *udev;
-	struct udev_device *dev = NULL;
 	struct stat st;
-	bool is_touchpad = false;
-
 	if (stat(devnode, &st) < 0)
 		return false;
 
-	udev = udev_new();
+	_unref_(udev) *udev = udev_new();
 	if (!udev)
-		goto out;
+		return false;
 
-	dev = udev_device_new_from_devnum(udev, 'c', st.st_rdev);
+	_unref_(udev_device) *dev = udev_device_new_from_devnum(udev, 'c', st.st_rdev);
 	if (!dev)
-		goto out;
+		return false;
 
-	is_touchpad = udev_device_get_property_value(dev, "ID_INPUT_TOUCHPAD");
-out:
-	if (dev)
-		udev_device_unref(dev);
-	udev_unref(udev);
-
-	return is_touchpad;
+	return udev_device_get_property_value(dev, "ID_INPUT_TOUCHPAD");
 }
 
 static inline void
@@ -765,21 +760,21 @@ setup_path(void)
 	const char *path = getenv("PATH");
 	char new_path[PATH_MAX];
 	const char *extra_path = LIBINPUT_TOOL_PATH;
-	char *builddir = builddir_lookup();
+	_autofree_ char *builddir = NULL;
 
+	builddir_lookup(&builddir);
 	snprintf(new_path,
 		 sizeof(new_path),
 		 "%s:%s",
 		 builddir ? builddir : extra_path,
 		 path ? path : "");
 	setenv("PATH", new_path, 1);
-	free(builddir);
 }
 
 int
 tools_exec_command(const char *prefix, int real_argc, char **real_argv)
 {
-	char *argv[64] = {NULL};
+	char *argv[64] = { NULL };
 	char executable[128];
 	const char *command;
 	int rc;
@@ -788,11 +783,7 @@ tools_exec_command(const char *prefix, int real_argc, char **real_argv)
 
 	command = real_argv[0];
 
-	rc = snprintf(executable,
-		      sizeof(executable),
-		      "%s-%s",
-		      prefix,
-		      command);
+	rc = snprintf(executable, sizeof(executable), "%s-%s", prefix, command);
 	if (rc >= (int)sizeof(executable)) {
 		fprintf(stderr, "Failed to assemble command.\n");
 		return EXIT_FAILURE;
@@ -807,9 +798,7 @@ tools_exec_command(const char *prefix, int real_argc, char **real_argv)
 	rc = execvp(executable, argv);
 	if (rc) {
 		if (errno == ENOENT) {
-			fprintf(stderr,
-				"libinput: %s is not installed\n",
-				command);
+			fprintf(stderr, "libinput: %s is not installed\n", command);
 			return EXIT_INVALID_USAGE;
 		}
 		fprintf(stderr,
@@ -842,7 +831,11 @@ sprintf_event_codes(char *buf, size_t sz, struct quirks *quirks, enum quirk q)
 
 		const char *name = libevdev_event_code_get_name(type, code);
 
-		printed = snprintf(buf + off, sz - off, "%c%s;", enable ? '+' : '-', name);
+		printed = snprintf(buf + off,
+				   sz - off,
+				   "%c%s;",
+				   enable ? '+' : '-',
+				   name);
 		assert(printed != -1);
 		off += printed;
 	}
@@ -868,7 +861,11 @@ sprintf_input_props(char *buf, size_t sz, struct quirks *quirks, enum quirk q)
 
 		const char *name = libevdev_property_get_name(prop);
 
-		printed = snprintf(buf + off, sz - off, "%c%s;", enable ? '+' : '-', name);
+		printed = snprintf(buf + off,
+				   sz - off,
+				   "%c%s;",
+				   enable ? '+' : '-',
+				   name);
 		assert(printed != -1);
 		off += printed;
 	}
@@ -882,10 +879,9 @@ tools_list_device_quirks(struct quirks_context *ctx,
 {
 	char buf[256];
 
-	struct quirks *quirks;
 	enum quirk q;
 
-	quirks = quirks_fetch_for_device(ctx, device);
+	_unref_(quirks) *quirks = quirks_fetch_for_device(ctx, device);
 	if (!quirks)
 		return;
 
@@ -900,7 +896,7 @@ tools_list_device_quirks(struct quirks_context *ctx,
 			snprintf(buf, sizeof(buf), "%s=%d", name, b ? 1 : 0);
 			callback(userdata, buf);
 		}
-	} while(++q < _QUIRK_LAST_MODEL_QUIRK_);
+	} while (++q < _QUIRK_LAST_MODEL_QUIRK_);
 
 	q = QUIRK_ATTR_SIZE_HINT;
 	do {
@@ -919,13 +915,23 @@ tools_list_device_quirks(struct quirks_context *ctx,
 			case QUIRK_ATTR_SIZE_HINT:
 			case QUIRK_ATTR_RESOLUTION_HINT:
 				quirks_get_dimensions(quirks, q, &dim);
-				snprintf(buf, sizeof(buf), "%s=%zdx%zd", name, dim.x, dim.y);
+				snprintf(buf,
+					 sizeof(buf),
+					 "%s=%zdx%zd",
+					 name,
+					 dim.x,
+					 dim.y);
 				callback(userdata, buf);
 				break;
 			case QUIRK_ATTR_TOUCH_SIZE_RANGE:
 			case QUIRK_ATTR_PRESSURE_RANGE:
 				quirks_get_range(quirks, q, &r);
-				snprintf(buf, sizeof(buf), "%s=%d:%d", name, r.upper, r.lower);
+				snprintf(buf,
+					 sizeof(buf),
+					 "%s=%d:%d",
+					 name,
+					 r.upper,
+					 r.lower);
 				callback(userdata, buf);
 				break;
 			case QUIRK_ATTR_PALM_SIZE_THRESHOLD:
@@ -952,6 +958,7 @@ tools_list_device_quirks(struct quirks_context *ctx,
 				break;
 			case QUIRK_ATTR_USE_VELOCITY_AVERAGING:
 			case QUIRK_ATTR_TABLET_SMOOTHING:
+			case QUIRK_ATTR_IS_VIRTUAL:
 				quirks_get_bool(quirks, q, &b);
 				snprintf(buf, sizeof(buf), "%s=%d", name, b);
 				callback(userdata, buf);
@@ -969,7 +976,5 @@ tools_list_device_quirks(struct quirks_context *ctx,
 				break;
 			}
 		}
-	} while(++q < _QUIRK_LAST_ATTR_QUIRK_);
-
-	quirks_unref(quirks);
+	} while (++q < _QUIRK_LAST_ATTR_QUIRK_);
 }
